package com.dfplibrary;

import android.content.Context;
import android.util.Log;

import androidx.annotation.NonNull;

import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.ReadableMapKeySetIterator;
import com.facebook.react.views.view.ReactViewGroup;
import com.google.android.gms.ads.AdListener;
import com.google.android.gms.ads.AdSize;
import com.google.android.gms.ads.LoadAdError;
import com.google.android.gms.ads.admanager.AdManagerAdRequest;
import com.google.android.gms.ads.admanager.AdManagerAdView;
import com.google.android.gms.ads.admanager.AppEventListener;

import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;

public class RNDfpBannerView extends ReactViewGroup implements AppEventListener {
    private AtomicBoolean loading = new AtomicBoolean( false );
    private String adUnitID = null;
    private AdSize adSize = null;
    private ReadableArray testDevices = null;
    private RNDfpBannerListener listener;
    private AdManagerAdView adView;
    private ReadableMap targeting;

    public RNDfpBannerView(Context context ){
        super( context );
    }

    public RNDfpBannerView(Context context, RNDfpBannerListener listener ){
        super( context );
        this.listener = listener;
    }

    public void setAdUnitId( String adUnitId ){
        this.adUnitID = adUnitId;
    }

    public void setAdSize( AdSize adSize ){
        this.adSize = adSize;
    }

    public void setTestDevices( ReadableArray testDevices ){
        this.testDevices = testDevices;
    }

    private AdManagerAdRequest initRequest(){
        AdManagerAdRequest.Builder request = new AdManagerAdRequest.Builder();
        if(targeting != null) {
            try{
                Map<String, Object> customTargeting = targeting.toHashMap();
                for(String key : customTargeting.keySet()){
                    try {
                        request.addCustomTargeting(key, (String) customTargeting.get(key));
                    } catch(ClassCastException | NullPointerException e) {
                        e.printStackTrace();
                    }
                }
            } catch(Exception e){
                e.printStackTrace();
            }
        }
        return request.build();
    }

    public void loadBanner(){
        try{
            if(!loading.get()){
                loading.set( true );
                createPublisherAdView( );
                if( listener != null ){
                    listener.onSizeChanged( getBannerId( ), adSize );
                }
                AdManagerAdRequest request = initRequest( );
                adView.loadAd( request );
            }
         } catch( Exception | NoSuchMethodError | NoClassDefFoundError e ){
             log( "onError");
             if( listener != null ){
                 listener.onError( getBannerId() );
             }
         }
    }

    private void updateAdPosition(){
        int width = adView.getAdSize( ).getWidthInPixels( getContext() );
        int height = adView.getAdSize( ).getHeightInPixels( getContext() );
        adView.measure( width, height );
        adView.layout( adView.getLeft( ), adView.getTop( ), adView.getLeft( ) + width, adView.getTop( ) + height );
    }

    protected void createPublisherAdView(){
        if( adView == null ){
            adView = new AdManagerAdView( getContext( ) );
            try {
                adView.setAdSizes(adSize);
                adView.setAdUnitId(adUnitID);
                adView.setAppEventListener(this);
                adView.setAdListener(new AdListener() {
                    @Override
                    public void onAdLoaded() {
                        log("onAdLoaded");
                        updateAdPosition();
                        if (listener != null) {
                            listener.onGoogleAdLoaded(getBannerId());
                        }
                        loading.set(false);
                    }

                    @Override
                    public void onAdFailedToLoad(@NonNull LoadAdError loadAdError) {
                        super.onAdFailedToLoad(loadAdError);
                        log("onAdFailedToLoad -> " + loadAdError.getCode());
                        if (listener != null) {
                            listener.onGoogleAdFailed(getBannerId(), loadAdError.getCode());
                        }
                        loading.set(false);
                    }

                    @Override
                    public void onAdOpened() {
                        if (listener != null) {
                            listener.onGoogleAdOpened(getBannerId());
                        }
                    }

                    @Override
                    public void onAdClosed() {
                        if (listener != null) {
                            listener.onGoogleAdClosed(getBannerId());
                        }
                    }
                });
            } catch (Exception e){
                e.printStackTrace();
                if (listener != null) {
                    listener.onError(getBannerId());
                }
            }
            addView( adView );
        }
    }

    @Override
    public void onAppEvent( String s, String s1 ){
        log( "onAppEvent -> s: " + s + " s1: " + s1);
        if( listener != null ){
            listener.onAppEventListener( getBannerId(), s, s1 );
        }
    }

    private int getBannerId(){
        return getId();
    }

    private void log( String message ){
        Log.d( "DFPBANNER", message );
    }
    public void setTargeting(ReadableMap map) {
        this.targeting = map;
    }
}
