package com.dfplibrary;

import android.util.Log;

import androidx.annotation.NonNull;

import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.google.android.gms.ads.AdRequest;
import com.google.android.gms.ads.LoadAdError;
import com.google.android.gms.ads.admanager.AdManagerAdRequest;
import com.google.android.gms.ads.admanager.AdManagerInterstitialAd;
import com.google.android.gms.ads.admanager.AdManagerInterstitialAdLoadCallback;
import com.google.android.gms.ads.interstitial.InterstitialAd;

import java.util.HashMap;
import java.util.Map;

public class RNDfpInterstitialAdModule extends ReactContextBaseJavaModule {

    private static final String TAG = "DFPInterstitial";
    private static final String REACT_CLASS = "RNDFPInterstitial";

    private InterstitialAd publisherInterstitialAd;
    private Promise loadAdFromConfigPromise = null;
    private boolean isLoading = false;


    @Override
    public String getName() {
        return REACT_CLASS;
    }

    public RNDfpInterstitialAdModule(ReactApplicationContext reactContext) {
        super(reactContext);
    }

    private void savedPromiseReject(String error){
        if(loadAdFromConfigPromise != null){
            loadAdFromConfigPromise.reject(new Exception(error));
            loadAdFromConfigPromise = null;
        }
    }

    private void onInterstitialFailed(int code){
        if(loadAdFromConfigPromise != null){
            loadAdFromConfigPromise.reject( new Exception( "DFP - Fail to load interstitial, error -> " + code ) );
        }
        isLoading = false;
    }

    private void onInterstitialLoaded(){
        if(loadAdFromConfigPromise != null){
            loadAdFromConfigPromise.resolve( "DFP - Interstitial loaded" );
        }
        isLoading = false;
    }

    private AdManagerAdRequest getPublisherRequest(ReadableMap map){
        AdManagerAdRequest.Builder publisherAdRequest = new AdManagerAdRequest.Builder();
        try {
            if (map.hasKey("targeting")) {
                HashMap<String, Object> targeting = map.getMap("targeting").toHashMap();
                for (String key : targeting.keySet()) {
                    try {
                        String value = (String) targeting.get(key);
                        publisherAdRequest.addCustomTargeting(key, value);
                    } catch (ClassCastException | NullPointerException e) {
                        e.printStackTrace();
                    }
                }
            }
        } catch(Exception e){
            e.printStackTrace();
        }
        return publisherAdRequest.build();
    }

    @ReactMethod
    public void loadAdFromAdUnitId(final ReadableMap map, final Promise promise ) {
        if(!isLoading) {
            isLoading = true;
            if (map != null) {
                Log.d(TAG, map.toString());
                if (map.hasKey("adUnitId") && map.getString("adUnitId") != null && !map.getString("adUnitId").equals("")) {
                    loadAdFromConfigPromise = promise;
                    final AdManagerAdRequest request = getPublisherRequest(map);
                    executeRunnable( new Runnable() {
                        @Override
                        public void run () {
                            try {
                                AdManagerInterstitialAd.load(
                                        getCurrentActivity(),
                                        map.getString("adUnitId"),
                                        request,
                                        new AdManagerInterstitialAdLoadCallback() {
                                            @Override
                                            public void onAdLoaded(@NonNull AdManagerInterstitialAd
                                                                           adManagerInterstitialAd) {
                                                super.onAdLoaded(adManagerInterstitialAd);
                                                publisherInterstitialAd = adManagerInterstitialAd;
                                                onInterstitialLoaded();
                                            }

                                            @Override
                                            public void onAdFailedToLoad(@NonNull LoadAdError loadAdError) {
                                                super.onAdFailedToLoad(loadAdError);
                                                publisherInterstitialAd = null;
                                                onInterstitialFailed(loadAdError.getCode());
                                            }
                                        }
                                );
                            } catch(Exception e){
                                isLoading = false;
                                promise.reject(e);
                            }
                        }
                    });
                } else {
                    isLoading = false;
                    promise.reject( new Exception( "Ad unit id must be set" ) );
                }
            } else {
                isLoading = false;
                promise.reject( new Exception( "Map can't be null" ) );
            }
        } else {
            promise.reject( new Exception( "We are loading another interstitial" ) );
        }
    }

    @ReactMethod
    public void showAd() {
        executeRunnable( new Runnable() {
            @Override
            public void run () {
                if(getCurrentActivity() != null){
                    publisherInterstitialAd.show(getCurrentActivity());
                } else {
                    Log.d( TAG, "activity is null" );
                }
            }
        });
    }

    private void executeRunnable(Runnable toRun){
        try {
            getCurrentActivity().runOnUiThread(toRun);
        } catch (Exception e){
            savedPromiseReject("current activity can't be null");
        }
    }

    @javax.annotation.Nullable
    @Override
    public Map<String, Object> getConstants() {
        final Map<String, Object> constants = new HashMap<>();
        constants.put("simulatorId", AdRequest.DEVICE_ID_EMULATOR);
        return constants;
    }
}