#import "RNDFPBannerView.h"

#if __has_include(<React/RCTBridgeModule.h>)
#import <React/RCTBridgeModule.h>
#import <React/UIView+React.h>
#import <React/RCTLog.h>
#import <React/RCTConvert.h>
#import <React/RCTUtils.h>
#else
#import "RCTBridgeModule.h"
#import "UIView+React.h"
#import "RCTLog.h"
#import "RCTConvert.h"
#import "RCTUtils.h"
#endif

@implementation RNDFPBannerView {
    GAMBannerView  *_bannerView;
}

- (void)insertReactSubview:(UIView *)view atIndex:(NSInteger)atIndex
{
    [super insertReactSubview:view atIndex:atIndex];
    RCTLogError(@"DFP Banner cannot have any subviews");
    return;
}

- (void)removeReactSubview:(UIView *)subview
{
    [super removeReactSubview:subview];
    RCTLogError(@"DFP Banner cannot have any subviews");
    return;
}

- (GADAdSize)getAdSizeFromString:(NSString *)bannerSize
{
    if ([bannerSize isEqualToString:@"banner"]) {
        return GADAdSizeBanner;
    } else if ([bannerSize isEqualToString:@"largeBanner"]) {
        return GADAdSizeLargeBanner;
    } else if ([bannerSize isEqualToString:@"mediumRectangle"]) {
        return GADAdSizeMediumRectangle;
    } else if ([bannerSize isEqualToString:@"fullBanner"]) {
        return GADAdSizeFullBanner;
    } else if ([bannerSize isEqualToString:@"leaderboard"]) {
        return GADAdSizeLeaderboard;
    } else {
        return GADAdSizeBanner;
    }
}

-(void)loadBanner {
    if (_adUnitID && (_bannerSize || _dimensions || _adSizes)) {
        GADAdSize size;
        NSMutableArray *validAdSizes;

        if (_dimensions) {
            NSNumber *width = [RCTConvert NSNumber:_dimensions[@"width"]];
            NSNumber *height = [RCTConvert NSNumber:_dimensions[@"height"]];

            CGFloat widthVal = [width doubleValue];
            CGFloat heightVal = [height doubleValue];

            CGSize cgSize = CGSizeMake(widthVal, heightVal);

            size = GADAdSizeFromCGSize(cgSize);
        } else {
            if (_adSizes) {
                validAdSizes = [[NSMutableArray alloc] init];
                for (id anAdSize in _adSizes) {
                    GADAdSize aSize;

                    // BannerSize
                    if ([anAdSize isKindOfClass:[NSString class]]) {
                        NSString *stringAdSize = (NSString *)anAdSize;

                        aSize = [self getAdSizeFromString:stringAdSize];

                        [validAdSizes addObject:NSValueFromGADAdSize(aSize)];

                        // Set size to the first one in the list
                        if (CGSizeEqualToSize(CGSizeFromGADAdSize(size), CGSizeMake(0.0, 0.0))) {
                            size = aSize;
                        }
                    }

                    // Dimensions
                    if ([anAdSize isKindOfClass:[NSDictionary class]]) {
                        NSDictionary *dictionaryAdSize = (NSDictionary *)anAdSize;

                        NSNumber *width = [RCTConvert NSNumber:dictionaryAdSize[@"width"]];
                        NSNumber *height = [RCTConvert NSNumber:dictionaryAdSize[@"height"]];

                        CGFloat widthVal = [width doubleValue];
                        CGFloat heightVal = [height doubleValue];

                        CGSize cgSize = CGSizeMake(widthVal, heightVal);

                        aSize = GADAdSizeFromCGSize(cgSize);

                        [validAdSizes addObject:NSValueFromGADAdSize(aSize)];

                        // Set size to the first one in the list
                        if (CGSizeEqualToSize(CGSizeFromGADAdSize(size), CGSizeMake(0.0, 0.0))) {
                            size = aSize;
                        }
                    }
                }
            } else {
                size = [self getAdSizeFromString:_bannerSize];
            }
        }

        _bannerView = [[GAMBannerView alloc] initWithAdSize:size];
        [_bannerView setAppEventDelegate:self]; //added Admob event dispatch listener
        // if(!CGRectEqualToRect(self.bounds, _bannerView.bounds)) {
            if (self.onSizeChange) {
                self.onSizeChange(@{
                                    @"width": [NSNumber numberWithFloat: _bannerView.bounds.size.width],
                                    @"height": [NSNumber numberWithFloat: _bannerView.bounds.size.height]
                                    });
            }
        // }
        _bannerView.delegate = self;
        _bannerView.adSizeDelegate = self;
        _bannerView.adUnitID = _adUnitID;
        _bannerView.rootViewController = RCTPresentedViewController();

        if (validAdSizes) {
          _bannerView.validAdSizes = validAdSizes;
        }

        GAMRequest *request = [GAMRequest request];
        if(_testDeviceID) {
            GADRequestConfiguration *requestConfiguration = GADMobileAds.sharedInstance.requestConfiguration;
            if([_testDeviceID isEqualToString:@"EMULATOR"]) {
                requestConfiguration.testDeviceIdentifiers = @[GADSimulatorID];
            } else {
                requestConfiguration.testDeviceIdentifiers = @[_testDeviceID];
            }
        }
        
        if (_targeting) {
            [request setCustomTargeting:_targeting];
        }

        [_bannerView loadRequest:request];
    }
}

- (void)adView:(GAMBannerView *)banner
didReceiveAppEvent:(NSString *)name
      withInfo:(NSString *)info {
    NSLog(@"Received app event (%@, %@)", name, info);
    NSMutableDictionary *myDictionary = [[NSMutableDictionary alloc] init];
    myDictionary[name] = info;
    if (self.onAdmobDispatchAppEvent) {
        self.onAdmobDispatchAppEvent(@{ name: info });
    }
}

- (void)didSetProps:(NSArray<NSString *> *)changedProps
{
    if ([changedProps containsObject:@"adSizes"] ||
        [changedProps containsObject:@"dimensions"] ||
        [changedProps containsObject:@"bannerSize"] ||
        [changedProps containsObject:@"adUnitID"] ||
        [changedProps containsObject:@"testDeviceID"] ||
        [changedProps containsObject:@"targeting"]) {
        
        if (_bannerView) {
            [_bannerView removeFromSuperview];
        }
        [self loadBanner];
    }
}

-(void)layoutSubviews
{
    [super layoutSubviews];

    _bannerView.frame = CGRectMake(
                                   self.bounds.origin.x,
                                   self.bounds.origin.y,
                                   _bannerView.frame.size.width,
                                   _bannerView.frame.size.height);
    [self addSubview:_bannerView];
}

- (void)removeFromSuperview
{
    [super removeFromSuperview];
}

/// Called before the ad view changes to the new size.
- (void)adView:(GAMBannerView *)bannerView
willChangeAdSizeTo:(GADAdSize)size {
    if (self.onWillChangeAdSizeTo) {
        // bannerView calls this method on its adSizeDelegate object before the banner updates it size,
        // allowing the application to adjust any views that may be affected by the new ad size.
        self.onWillChangeAdSizeTo(@{});
    }

}

/// Tells the delegate an ad request loaded an ad.
- (void)bannerViewDidReceiveAd:(GAMBannerView *)adView {
    if (self.onAdViewDidReceiveAd) {
        self.onAdViewDidReceiveAd(@{});
    }
    if (self.onSizeChange) {
        self.onSizeChange(@{
                            @"width": [NSNumber numberWithFloat: _bannerView.bounds.size.width],
                            @"height": [NSNumber numberWithFloat: _bannerView.bounds.size.height]
                            });
    }
}

/// Tells the delegate an ad request failed.
- (void)bannerView:(GAMBannerView *)adView
didFailToReceiveAdWithError:(NSError *)error {
    if (self.onDidFailToReceiveAdWithError) {
        self.onDidFailToReceiveAdWithError(@{ @"error": [error localizedDescription] });
    }
}

/// Tells the delegate that a full screen view will be presented in response
/// to the user clicking on an ad.
- (void)bannerViewWillPresentScreen:(GAMBannerView *)adView {
    if (self.onAdViewWillPresentScreen) {
        self.onAdViewWillPresentScreen(@{});
    }
}

/// Tells the delegate that the full screen view will be dismissed.
- (void)bannerViewWillDismissScreen:(GAMBannerView *)adView {
    if (self.onAdViewWillDismissScreen) {
        self.onAdViewWillDismissScreen(@{});
    }
}

/// Tells the delegate that the full screen view has been dismissed.
- (void)bannerViewDidDismissScreen:(GAMBannerView *)adView {
    if (self.onAdViewDidDismissScreen) {
        self.onAdViewDidDismissScreen(@{});
    }
}

@end
