//
//  RNDFPInterstitial.m
//  RNDfp
//
//  Created by Bub on 7/28/17.
//  Copyright © 2017 Facebook. All rights reserved.
//

#import "RNDFPInterstitial.h"

@implementation RNDFPInterstitial {
    RCTPromiseResolveBlock globalResolve;
    RCTPromiseRejectBlock globalReject;
    RCTResponseSenderBlock globalClose;
}

- (dispatch_queue_t)methodQueue
{
    return dispatch_get_main_queue();
}

RCT_EXPORT_MODULE();

RCT_REMAP_METHOD(loadAdFromAdUnitId, loadAdFromAdUnitId:(NSDictionary *)config resolve:(RCTPromiseResolveBlock)resolve reject:(RCTPromiseRejectBlock)reject)
{
    if (globalResolve) {
        reject(@"Error",@"Another promise pending discard",nil);
    } else {
        globalResolve = resolve;
        globalReject = reject;
    
        NSLog(@"Loading Interstitial ad...");
        GAMRequest *request = [GAMRequest request];
        
        NSString *adUnitId = [config objectForKey:@"adUnitId"];
        
        NSDictionary *targeting = [config objectForKey:@"targeting"];
        if (targeting) {
            [request setCustomTargeting:targeting];
        }
        
        [GAMInterstitialAd loadWithAdManagerAdUnitID:adUnitId
                                    request:request
                          completionHandler:^(GAMInterstitialAd *ad, NSError *error) {
            if (error) {
              NSLog(@"Failed to load interstitial ad with error: %@", [error localizedDescription]);
                self->globalReject(@"Error",[error description],nil);
                self->globalResolve = nil;
                self->globalReject = nil;
                self->globalClose = nil;
              return;
            }
            NSLog(@"DFP interstitial loaded");
            self.interstitial = ad;
            self.interstitial.fullScreenContentDelegate = self;
                
            self->globalResolve(nil);
                
            self->globalResolve = nil;
            self->globalReject = nil;
        }];
    }
}

RCT_REMAP_METHOD(showAd, showAd:(RCTResponseSenderBlock)closeCallback)
{
    NSLog(@"Checking Interstitial ad...");
    globalClose = closeCallback;
    if (self.interstitial) {
        NSLog(@"Showing Interstitial ad...");
        UIViewController * _root = RCTPresentedViewController();
        [self.interstitial presentFromRootViewController:_root];
    } else {
        NSLog(@"Interstitial Ad wasn't ready");
    }
}

-(void)adWillDismissFullScreenContent:(GAMInterstitialAd *)ad {
    if (globalClose) {
        globalClose(@[]);
        globalClose = nil;
    }
}

@end
