import React from "react";
import PropTypes from "prop-types";
import { requireNativeComponent } from "react-native";
export default class DFPBanner extends React.Component {
  static propTypes = {
    onDidFailToReceiveAdWithError: PropTypes.func,
    onAdmobDispatchAppEvent: PropTypes.func,
    onAdViewDidReceiveAd: PropTypes.func,
    onAdViewWillPresentScreen: PropTypes.func,
    onAdViewWillDismissScreen: PropTypes.func,
    onAdViewDidDismissScreen: PropTypes.func,
    /**
     * banner (320x50, Standard Banner for Phones and Tablets)
     * largeBanner (320x100, Large Banner for Phones and Tablets)
     * mediumRectangle (300x250, IAB Medium Rectangle for Phones and Tablets)
     * fullBanner (468x60, IAB Full-Size Banner for Tablets)
     * leaderboard (728x90, IAB Leaderboard for Tablets)
     * smartBannerPortrait (Screen width x 32|50|90, Smart Banner for Phones and Tablets)
     * smartBannerLandscape (Screen width x 32|50|90, Smart Banner for Phones and Tablets)
     */
    bannerSize: PropTypes.string,
    /**
     * Custom banner size (instead of using bannerSize)
     */
    dimensions: PropTypes.shape({
      height: PropTypes.number,
      width: PropTypes.number
    }),
    /**
     * Array of some combination of bannerSize and dimensions that are valid for the ad
     * Example: ['mediumRectangle', { width: 320, height: 400 }, 'smartBannerPortrait']
     */
    adSizes: PropTypes.array,
    adUnitID: PropTypes.string,
    testDeviceID: PropTypes.string,
    targeting: PropTypes.object
  };
  constructor() {
    super();
    this.onSizeChange = this.onSizeChange.bind(this);
    this.onDidFailToReceiveAdWithError = this.onDidFailToReceiveAdWithError.bind(this);
    this.onAdmobDispatchAppEvent = this.onAdmobDispatchAppEvent.bind(this);
    this.state = {
      style: {}
    };
  }
  onSizeChange({
    nativeEvent
  }) {
    const {
      height,
      width
    } = nativeEvent;
    this.setState({
      style: {
        width,
        height
      }
    });
  }
  onDidFailToReceiveAdWithError({
    nativeEvent
  }) {
    const {
      onDidFailToReceiveAdWithError
    } = this.props;
    if (typeof onDidFailToReceiveAdWithError === "function") {
      onDidFailToReceiveAdWithError(nativeEvent.error);
    }
  }
  onAdmobDispatchAppEvent(event) {
    const {
      onAdmobDispatchAppEvent
    } = this.props;
    if (typeof onAdmobDispatchAppEvent === "function") {
      onAdmobDispatchAppEvent(event);
    }
  }
  render() {
    const {
      adUnitID,
      testDeviceID,
      dimensions,
      onAdViewDidReceiveAd,
      onAdViewWillPresentScreen,
      onAdViewWillDismissScreen,
      onAdViewDidDismissScreen,
      targeting
    } = this.props;
    let {
      bannerSize,
      adSizes
    } = this.props;
    const {
      style
    } = this.state;

    // Dimensions gets highest priority
    if (dimensions && dimensions.width && dimensions.height) {
      bannerSize = undefined;
      adSizes = undefined;
    }

    // AdSizes gets second priority
    if (adSizes && adSizes.length > 0) {
      bannerSize = undefined;
    }

    // Default to something if nothing is set
    if (!bannerSize && (!dimensions || !dimensions.width || !dimensions.height) && (!adSizes || !adSizes.length > 0)) {
      bannerSize = "smartBannerPortrait";
    }
    return /*#__PURE__*/React.createElement(RNBanner, {
      style: style,
      onSizeChange: this.onSizeChange,
      onAdViewDidReceiveAd: onAdViewDidReceiveAd,
      onDidFailToReceiveAdWithError: this.onDidFailToReceiveAdWithError,
      onAdViewWillPresentScreen: onAdViewWillPresentScreen,
      onAdViewWillDismissScreen: onAdViewWillDismissScreen,
      onAdViewDidDismissScreen: onAdViewDidDismissScreen,
      onAdmobDispatchAppEvent: this.onAdmobDispatchAppEvent,
      testDeviceID: testDeviceID,
      adUnitID: adUnitID,
      adSizes: adSizes,
      dimensions: dimensions,
      bannerSize: bannerSize,
      targeting: targeting
    });
  }
}
const RNBanner = requireNativeComponent("RNDFPBanner", DFPBanner);
//# sourceMappingURL=RNDFPBanner.js.map